"use strict";
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var constants_1 = require("../../utilities/constants");
var imports_1 = require("../../utilities/imports");
var jsx_1 = require("../../utilities/jsx");
exports.extensions = constants_1.typescript.extensions;
exports.options = {
    relative: constants_1.typescript.options.relative,
    renameFrom: {
        name: 'renameFrom',
        type: 'string',
        description: 'The component name to target. (ex: Card)',
        required: true
    },
    renameTo: {
        name: 'renameTo',
        type: 'string',
        description: 'The new component name.',
        required: true
    },
    renamePropsFrom: {
        name: 'renamePropsFrom',
        type: 'string',
        description: 'The target component props to rename. (ex: CardProps)',
        required: true
    },
    renamePropsTo: {
        name: 'renamePropsTo',
        type: 'string',
        description: 'The new component props name.',
        required: true
    }
};
function transformer(fileInfo, _a, options) {
    var j = _a.jscodeshift;
    var source = j(fileInfo.source);
    if (!options.relative &&
        !(0, imports_1.hasImportDeclaration)(j, source, '@shopify/polaris')) {
        return fileInfo.source;
    }
    var sourcePaths = (0, imports_1.normalizeImportSourcePaths)(j, source, {
        relative: options.relative,
        from: options.renameFrom,
        to: options.renameTo
    });
    if (!sourcePaths)
        return;
    // If renameFrom component name is not imported, exit
    if (!(0, imports_1.hasImportSpecifier)(j, source, options.renameFrom, sourcePaths.from) &&
        !(0, imports_1.hasImportSpecifier)(j, source, options.renamePropsFrom, sourcePaths.from)) {
        return fileInfo.source;
    }
    var hasExistingJsx = false;
    var hasExistingIdentifier = false;
    // If local renameTo is already used in the file, exit
    source.findJSXElements(options.renameTo).forEach(function (element) {
        (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
        hasExistingJsx = true;
    });
    if (hasExistingJsx)
        return source.toSource();
    // If renameTo is already used as an identifier, exit
    source
        .find(j.Identifier)
        .filter(function (path) {
        return path.node.name === options.renameTo ||
            path.node.name === options.renamePropsTo;
    })
        .forEach(function (path) {
        if (path.node.type !== 'Identifier')
            return;
        (0, jsx_1.insertCommentBefore)(j, path, constants_1.POLARIS_MIGRATOR_COMMENT);
        hasExistingIdentifier = true;
    });
    if (hasExistingIdentifier)
        return source.toSource();
    var localElementName = (0, imports_1.getImportSpecifierName)(j, source, options.renameFrom, sourcePaths.from) ||
        options.renameFrom;
    var localElementTypeName = (0, imports_1.getImportSpecifierName)(j, source, options.renamePropsFrom, sourcePaths.from);
    source.find(j.JSXElement).forEach(function (element) {
        var _a, _b, _c;
        // Handle self-closing elements
        if (element.node.openingElement.selfClosing === true &&
            element.node.openingElement.name.type === 'JSXIdentifier' &&
            element.node.openingElement.name.name === localElementName) {
            element.node.openingElement.name.name = options.renameTo;
        }
        // Handle simple elements
        if (element.node.openingElement.name.type === 'JSXIdentifier' &&
            element.node.openingElement.name.name === localElementName) {
            (0, jsx_1.replaceJSXElement)(j, element, options.renameTo);
            return;
        }
        // Handle self-closing compound elements
        if (element.node.openingElement.selfClosing === true &&
            element.node.openingElement.name.type === 'JSXMemberExpression' &&
            element.node.openingElement.name.object.type === 'JSXIdentifier' &&
            element.node.openingElement.name.object.name === localElementName) {
            element.node.openingElement.name.object.name = options.renameTo;
        }
        // Handle compound elements
        if (element.node.openingElement.name.type === 'JSXMemberExpression' &&
            element.node.openingElement.name.object.type === 'JSXIdentifier' &&
            element.node.openingElement.name.object.name === localElementName &&
            ((_a = element.node.closingElement) === null || _a === void 0 ? void 0 : _a.name.type) === 'JSXMemberExpression' &&
            ((_b = element.node.closingElement) === null || _b === void 0 ? void 0 : _b.name.object.type) === 'JSXIdentifier' &&
            ((_c = element.node.closingElement) === null || _c === void 0 ? void 0 : _c.name.object.name) === localElementName) {
            element.node.openingElement.name.object.name = options.renameTo;
            element.node.closingElement.name.object.name = options.renameTo;
        }
    });
    // Find all references to the renameFrom component and replace them with renameTo
    source
        .find(j.Identifier)
        .filter(function (path) {
        return path.node.name === localElementName ||
            path.node.name === localElementTypeName;
    })
        .forEach(function (path) {
        if (path.node.type !== 'Identifier')
            return;
        if (path.node.name === localElementName &&
            path.parent.value.type !== 'MemberExpression') {
            path.node.name = options.renameTo;
        }
        if (path.node.name === localElementTypeName) {
            path.node.name = options.renamePropsTo;
        }
    });
    if (!(0, imports_1.hasImportSpecifier)(j, source, options.renameTo, sourcePaths.to)) {
        if (options.relative) {
            (0, imports_1.insertImportDeclaration)(j, source, options.renameTo, sourcePaths.to, sourcePaths.from);
        }
        else {
            (0, imports_1.insertImportSpecifier)(j, source, options.renameTo, sourcePaths.to);
        }
    }
    // Remove the renameFrom import
    if ((0, imports_1.hasImportSpecifier)(j, source, options.renameFrom, sourcePaths.from)) {
        (0, imports_1.removeImportSpecifier)(j, source, options.renameFrom, sourcePaths.from);
    }
    // Remove the renamePropsFrom import
    if (!(0, imports_1.hasImportSpecifiers)(j, source, sourcePaths.from)) {
        (0, imports_1.removeImportDeclaration)(j, source, sourcePaths.from);
    }
    return source.toSource();
}
exports["default"] = transformer;
