"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var sass_1 = require("../../utilities/sass");
var constants_1 = require("../../utilities/constants");
var matches_1 = require("../../utilities/matches");
exports.extensions = constants_1.scss.extensions;
exports.options = {
    url: {
        name: 'url',
        type: 'string',
        description: 'The @use rule URL to replace if unused.'
    }
};
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var url = options.url;
    if (!url) {
        throw new Error('Missing required options: url');
    }
    return {
        postcssPlugin: 'scss-remove-unused-at-use',
        Once: function (root) {
            root.walkAtRules('use', function (atRule) {
                var _a, _b, _c, _d, _e;
                var atUseParams = atRule.params.replace(/(\s|\n|\r\n?)+/g, ' ');
                var atUseURL = (_b = (_a = atUseParams.match(/(['"])(.+?)\1/)) === null || _a === void 0 ? void 0 : _a[2]) !== null && _b !== void 0 ? _b : '';
                var atUseAsNamespace = (_d = (_c = atUseParams
                    .match(/\sas\s+(.+?)(\s|$)/)) === null || _c === void 0 ? void 0 : _c[1]) === null || _d === void 0 ? void 0 : _d.trim();
                // Didn't find the target import in file
                if (!atUseURL || !(0, matches_1.matchesStringOrRegExp)(atUseURL, url)) {
                    return;
                }
                // const namespace = atUseParams.match(/as\s*((?:[^\s;])+)/)?.[1];
                var atUseURLNamespace = atUseURL.split('/').pop();
                var atUseNamespace = atUseAsNamespace || atUseURLNamespace;
                // Comment if the namespace is using an alias
                if (atUseNamespace === '*') {
                    atRule.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                    return;
                }
                // Remove target import
                if (!((_e = root.source) === null || _e === void 0 ? void 0 : _e.input.css.includes("".concat(atUseNamespace, ".")))) {
                    atRule.remove();
                }
            });
        }
    };
};
function scssRemoveUnusedImport(file, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(file.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = scssRemoveUnusedImport;
