"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.extensions = void 0;
var os_1 = __importDefault(require("os"));
var postcss_1 = __importDefault(require("postcss"));
var stylelint_1 = __importDefault(require("stylelint"));
var constants_1 = require("../../utilities/constants");
var crossPlatformNewlineRegExp = /\r\n?|\n/;
var polarisContextMsg = '-- generated by polaris-migrator DO NOT COPY';
exports.extensions = constants_1.scss.extensions;
var plugin = function () {
    return {
        postcssPlugin: 'styles-insert-stylelint-disable',
        Once: function (root, _a) {
            var result = _a.result;
            result.messages.forEach(function (_a) {
                var _b, _c, _d;
                var node = _a.node;
                // If a polaris ignore comment exists above the node already,
                // do nothing
                if ((_c = (_b = node.prev()) === null || _b === void 0 ? void 0 : _b.text) === null || _c === void 0 ? void 0 : _c.includes(polarisContextMsg)) {
                    return;
                }
                var isMultiline = crossPlatformNewlineRegExp.test(node.value) ||
                    crossPlatformNewlineRegExp.test(node.text) ||
                    crossPlatformNewlineRegExp.test(node.params);
                var commentText = "".concat(isMultiline ? 'stylelint-disable' : 'stylelint-disable-next-line', " ").concat(polarisContextMsg);
                var comment = createDisableComment(commentText, node.prev());
                node.before(comment);
                if (isMultiline) {
                    (node.type === 'atrule' && ((_d = node === null || node === void 0 ? void 0 : node.nodes) === null || _d === void 0 ? void 0 : _d.length)) ||
                        node.type === 'rule'
                        ? node.prepend(createCommentNode('stylelint-enable'))
                        : node.after(createCommentNode('stylelint-enable'));
                }
                deleteExtraNewlinesBeforeNode(node);
            });
            fillDescriptionlessDisables(result, root);
        }
    };
};
function transformer(file, _, options) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_b) {
            return [2 /*return*/, (0, postcss_1["default"])([
                    (0, stylelint_1["default"])({
                        config: {
                            "extends": [(_a = options.config) !== null && _a !== void 0 ? _a : '@shopify/stylelint-polaris']
                        }
                    }),
                    plugin(),
                ])
                    .process(file.source, {
                    from: file.path,
                    syntax: require('postcss-scss')
                })
                    .then(function (result) {
                    return result.css;
                })];
        });
    });
}
exports["default"] = transformer;
/**
 * Create a postcss comment node in the style:
 * `// ${text}`
 */
function createCommentNode(text) {
    return new postcss_1["default"].Comment({
        text: text,
        raws: {
            left: ' ',
            right: '',
            inline: true
        }
    });
}
/**
 * Create a new disable comment with the given text.
 * If the prevNode and the text are both disable-next-line
 * comments, they are combined into a single comment with their
 * description texts seperated by a comma.
 */
function createDisableComment(text, prevNode) {
    var _a, _b, _c;
    if ((prevNode === null || prevNode === void 0 ? void 0 : prevNode.type) !== 'comment' ||
        !prevNode.text.includes('stylelint-disable-next-line') ||
        !text.includes('stylelint-disable-next-line')) {
        return createCommentNode(text);
    }
    var prevDescription = (_c = (_b = (_a = prevNode.text) === null || _a === void 0 ? void 0 : _a.split('--')) === null || _b === void 0 ? void 0 : _b[1]) === null || _c === void 0 ? void 0 : _c.trim();
    var commentText = (prevDescription === null || prevDescription === void 0 ? void 0 : prevDescription.length)
        ? [text, prevDescription].join(', ')
        : text;
    prevNode.remove();
    return createCommentNode(commentText);
}
/**
 * Reduces the number of newline characters in a node's before
 * raws to just one. This is helpful to ensure that there is
 * only one newline between a disable-next-line comment and
 * the warning node.
 */
function deleteExtraNewlinesBeforeNode(node) {
    node.raws.before = "".concat(os_1["default"].EOL).concat(node.raws.before.replace(new RegExp(crossPlatformNewlineRegExp, 'g'), ''));
}
/**
 * If report-descriptionless-disables is enabled on stylelint,
 * add a generated description to disables that don't have one.
 */
function fillDescriptionlessDisables(result, root) {
    var _a;
    var stylelintResult = result === null || result === void 0 ? void 0 : result.stylelint;
    if ((_a = stylelintResult === null || stylelintResult === void 0 ? void 0 : stylelintResult.config) === null || _a === void 0 ? void 0 : _a.reportDescriptionlessDisables) {
        root.walkComments(function (comment) {
            if (comment.text.includes('stylelint-disable') &&
                !comment.text.includes('--')) {
                comment.before(createCommentNode([comment.text, polarisContextMsg].join(' ')));
                comment.remove();
            }
        });
    }
}
