"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.extensions = void 0;
var path_1 = __importDefault(require("path"));
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
var matches_1 = require("../../utilities/matches");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
function transformer(file, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(file.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
function plugin(options) {
    var _a, _b;
    if (options === void 0) { options = {}; }
    var replacementMaps;
    if (options.maps && (options.from || options.to || options.decl)) {
        throw new Error('--maps is not permitted with --from or --to');
    }
    if (options.maps) {
        var mapsPath = path_1["default"].resolve(process.cwd(), options.maps);
        replacementMaps = require(mapsPath)["default"];
    }
    else if (options.from && options.to) {
        replacementMaps = (_a = {},
            _a[options.decl || '/.+/'] = (_b = {},
                _b[options.from] = options.to,
                _b),
            _a);
    }
    else if (options.replacementMaps) {
        replacementMaps = options.replacementMaps;
    }
    if (!replacementMaps) {
        throw new Error('Unable to resolve the replacement maps');
    }
    var replacementMapsKeys = Object.keys(replacementMaps);
    return {
        postcssPlugin: 'styles-replace-custom-property',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed] || !replacementMaps)
                return;
            var matchedDecl = (0, matches_1.matchesStringOrRegExp)(decl.prop, replacementMapsKeys);
            if (!matchedDecl)
                return;
            var replacementMap = replacementMaps[matchedDecl.pattern.toString()];
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            parsedValue.walk(function (node) {
                if (!(0, sass_1.isSassFunction)('var', node))
                    return;
                for (var _i = 0, _a = node.nodes; _i < _a.length; _i++) {
                    var argNode = _a[_i];
                    if (argNode.type !== 'word' ||
                        !argNode.value.startsWith('--p-') ||
                        !(0, types_1.isKeyOf)(replacementMap, argNode.value)) {
                        continue;
                    }
                    var replacement = replacementMap[argNode.value];
                    if (replacement.startsWith('--')) {
                        argNode.value = replacement;
                        continue;
                    }
                    // @ts-expect-error - Reassign the type to word
                    node.type = 'word';
                    node.value = replacement;
                    break;
                }
            });
            decl.value = parsedValue.toString();
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
}
