"use strict";
exports.__esModule = true;
exports.replaceDisplayText = void 0;
var jsx_1 = require("../../../utilities/jsx");
var types_1 = require("../../../utilities/types");
var imports_1 = require("../../../utilities/imports");
var constants_1 = require("../../../utilities/constants");
var displayTextElements = new Set(['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p']);
var defaultDisplayTextElement = 'p';
var displayTextSizeMap = {
    small: 'headingLg',
    medium: 'headingXl',
    large: 'heading2xl',
    extraLarge: 'heading4xl'
};
var defaultDisplayTextSize = 'medium';
/**
 * Replace <DisplayText> with the <Text> component
 */
function replaceDisplayText(j, source, options) {
    var sourcePaths = (0, imports_1.normalizeImportSourcePaths)(j, source, {
        relative: options.relative,
        from: 'DisplayText',
        to: 'Text'
    });
    if (!sourcePaths)
        return;
    if (!(0, imports_1.hasImportSpecifier)(j, source, 'DisplayText', sourcePaths.from) &&
        !(0, imports_1.hasImportSpecifier)(j, source, 'DisplayTextProps', sourcePaths.from)) {
        return;
    }
    var localElementName = (0, imports_1.getImportSpecifierName)(j, source, 'DisplayText', sourcePaths.from) ||
        'DisplayText';
    var localElementTypeName = (0, imports_1.getImportSpecifierName)(j, source, 'DisplayTextProps', sourcePaths.from);
    var canInsertTextImport = false;
    var canRemoveDisplayTextImport = Boolean(!localElementTypeName);
    source.findJSXElements(localElementName).forEach(function (element) {
        var _a;
        var allAttributes = (_a = j(element).find(j.JSXOpeningElement).get().value
            .attributes) !== null && _a !== void 0 ? _a : [];
        if (allAttributes.some(function (attribute) { return attribute.type !== 'JSXAttribute'; })) {
            canRemoveDisplayTextImport = false;
            (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
            return;
        }
        var jsxAttributes = allAttributes;
        var elementAttribute = jsxAttributes.find(function (attribute) { return attribute.name.name === 'element'; });
        var elementValue = defaultDisplayTextElement;
        if (elementAttribute) {
            var elementAttributeValue = elementAttribute.value;
            if ((elementAttributeValue === null || elementAttributeValue === void 0 ? void 0 : elementAttributeValue.type) === 'StringLiteral' &&
                displayTextElements.has(elementAttributeValue.value)) {
                elementValue = elementAttributeValue.value;
            }
            else {
                canRemoveDisplayTextImport = false;
                (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
                return;
            }
        }
        var sizeAttribute = jsxAttributes.find(function (attribute) { return attribute.name.name === 'size'; });
        var sizeValue = defaultDisplayTextSize;
        if (sizeAttribute) {
            var sizeAttributeValue = sizeAttribute.value;
            if ((sizeAttributeValue === null || sizeAttributeValue === void 0 ? void 0 : sizeAttributeValue.type) === 'StringLiteral' &&
                (0, types_1.isKeyOf)(displayTextSizeMap, sizeAttributeValue.value)) {
                sizeValue = sizeAttributeValue.value;
            }
            else {
                canRemoveDisplayTextImport = false;
                (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
                return;
            }
        }
        canInsertTextImport = true;
        (0, jsx_1.replaceJSXElement)(j, element, 'Text');
        if (!(0, jsx_1.hasJSXAttribute)(j, element, 'size')) {
            (0, jsx_1.insertJSXAttribute)(j, element, 'variant', displayTextSizeMap[sizeValue]);
        }
        else {
            (0, jsx_1.replaceJSXAttributes)(j, element, 'size', 'variant', displayTextSizeMap[sizeValue]);
        }
        if (!(0, jsx_1.hasJSXAttribute)(j, element, 'element')) {
            (0, jsx_1.insertJSXAttribute)(j, element, 'as', elementValue);
        }
        else {
            (0, jsx_1.replaceJSXAttributes)(j, element, 'element', 'as', elementValue);
        }
    });
    source
        .find(j.Identifier)
        .filter(function (path) {
        return path.node.name === localElementName ||
            path.node.name === localElementTypeName;
    })
        .forEach(function (path) {
        if (path.node.type !== 'Identifier')
            return;
        if (path.parent.value.type !== 'ImportSpecifier') {
            canRemoveDisplayTextImport = false;
        }
        (0, jsx_1.insertCommentBefore)(j, path, constants_1.POLARIS_MIGRATOR_COMMENT);
        if (path.node.name === localElementName) {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: Text');
        }
        else {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: TextProps');
        }
    });
    if (canInsertTextImport) {
        if (!(0, imports_1.hasImportSpecifier)(j, source, 'Text', sourcePaths.to)) {
            if (options.relative) {
                (0, imports_1.insertImportDeclaration)(j, source, 'Text', sourcePaths.to, sourcePaths.from);
            }
            else {
                (0, imports_1.insertImportSpecifier)(j, source, 'Text', sourcePaths.to);
            }
        }
    }
    if (canRemoveDisplayTextImport) {
        if ((0, imports_1.hasImportSpecifier)(j, source, 'DisplayText', sourcePaths.from)) {
            (0, imports_1.removeImportSpecifier)(j, source, 'DisplayText', sourcePaths.from);
        }
        if (!(0, imports_1.hasImportSpecifiers)(j, source, sourcePaths.from)) {
            (0, imports_1.removeImportDeclaration)(j, source, sourcePaths.from);
        }
    }
}
exports.replaceDisplayText = replaceDisplayText;
