"use strict";
exports.__esModule = true;
exports.replaceOther = exports.components = void 0;
var jsx_1 = require("../../../utilities/jsx");
var imports_1 = require("../../../utilities/imports");
var constants_1 = require("../../../utilities/constants");
var types_1 = require("../../../utilities/types");
exports.components = {
    Heading: {
        validElements: new Set(['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p']),
        defaultElement: 'h2',
        as: 'h2',
        variant: 'headingMd'
    },
    Subheading: {
        validElements: new Set(['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p']),
        defaultElement: 'h3',
        as: 'h3',
        variant: 'headingXs'
    },
    Caption: {
        validElements: new Set(),
        defaultElement: undefined,
        as: 'p',
        variant: 'bodySm'
    },
    VisuallyHidden: {
        validElements: new Set(),
        defaultElement: undefined,
        as: 'span',
        variant: 'bodySm'
    }
};
function replaceOtherComponent(j, source, componentName, relative) {
    var _a = exports.components[componentName], variant = _a.variant, as = _a.as, validElements = _a.validElements, defaultElement = _a.defaultElement;
    var sourcePaths = (0, imports_1.normalizeImportSourcePaths)(j, source, {
        relative: relative,
        from: componentName,
        to: 'Text'
    });
    if (!sourcePaths)
        return;
    if (!(0, imports_1.hasImportSpecifier)(j, source, componentName, sourcePaths.from) &&
        !(0, imports_1.hasImportSpecifier)(j, source, "".concat(componentName, "Props"), sourcePaths.from)) {
        return;
    }
    var localElementName = (0, imports_1.getImportSpecifierName)(j, source, componentName, sourcePaths.from) ||
        componentName;
    var localElementTypeName = (0, imports_1.getImportSpecifierName)(j, source, "".concat(componentName, "Props"), sourcePaths.from);
    var canInsertTextImport = false;
    var canRemoveComponentImport = Boolean(!localElementTypeName);
    source.findJSXElements(localElementName).forEach(function (element) {
        var _a;
        var allAttributes = (_a = j(element).find(j.JSXOpeningElement).get().value
            .attributes) !== null && _a !== void 0 ? _a : [];
        if (allAttributes.some(function (attribute) { return attribute.type !== 'JSXAttribute'; })) {
            canRemoveComponentImport = false;
            (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
            return;
        }
        var jsxAttributes = allAttributes;
        var elementAttribute = jsxAttributes.find(function (attribute) { return attribute.name.name === 'element'; });
        var elementValue = defaultElement;
        if (elementAttribute) {
            var elementAttributeValue = elementAttribute.value;
            if ((elementAttributeValue === null || elementAttributeValue === void 0 ? void 0 : elementAttributeValue.type) === 'StringLiteral' &&
                validElements.has(elementAttributeValue.value)) {
                elementValue = elementAttributeValue.value;
            }
            else {
                canRemoveComponentImport = false;
                (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
                return;
            }
        }
        canInsertTextImport = true;
        (0, jsx_1.replaceJSXElement)(j, element, 'Text');
        (0, jsx_1.insertJSXAttribute)(j, element, 'variant', variant);
        if ((0, jsx_1.hasJSXAttribute)(j, element, 'element')) {
            (0, jsx_1.replaceJSXAttributes)(j, element, 'element', 'as', elementValue !== null && elementValue !== void 0 ? elementValue : as);
        }
        else {
            (0, jsx_1.insertJSXAttribute)(j, element, 'as', elementValue !== null && elementValue !== void 0 ? elementValue : as);
        }
        if (componentName === 'VisuallyHidden') {
            (0, jsx_1.insertJSXAttribute)(j, element, 'visuallyHidden');
        }
    });
    source
        .find(j.Identifier)
        .filter(function (path) {
        return path.node.name === localElementName ||
            path.node.name === localElementTypeName;
    })
        .forEach(function (path) {
        if (path.node.type !== 'Identifier')
            return;
        if (path.parent.value.type !== 'ImportSpecifier') {
            canRemoveComponentImport = false;
        }
        (0, jsx_1.insertCommentBefore)(j, path, constants_1.POLARIS_MIGRATOR_COMMENT);
        if (path.node.name === localElementName) {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: Text');
        }
        else {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: TextProps');
        }
    });
    if (canInsertTextImport) {
        if (!(0, imports_1.hasImportSpecifier)(j, source, 'Text', sourcePaths.to)) {
            if (relative) {
                (0, imports_1.insertImportDeclaration)(j, source, 'Text', sourcePaths.to, sourcePaths.from);
            }
            else {
                (0, imports_1.insertImportSpecifier)(j, source, 'Text', sourcePaths.to);
            }
        }
    }
    if (canRemoveComponentImport) {
        if ((0, imports_1.hasImportSpecifier)(j, source, componentName, sourcePaths.from)) {
            (0, imports_1.removeImportSpecifier)(j, source, componentName, sourcePaths.from);
        }
        if (!(0, imports_1.hasImportSpecifiers)(j, source, sourcePaths.from)) {
            (0, imports_1.removeImportDeclaration)(j, source, sourcePaths.from);
        }
    }
}
/**
 * Replace <Heading>, <Subheading>, <Caption>, and <VisuallyHidden> with the <Text> component
 */
function replaceOther(j, source, options) {
    if (options.componentName && !(0, types_1.isKeyOf)(exports.components, options.componentName)) {
        throw new Error("Unsupported component name: ".concat(options.componentName));
    }
    if (options.componentName) {
        replaceOtherComponent(j, source, options.componentName, options.relative);
    }
    else {
        Object.keys(exports.components).forEach(function (componentName) {
            return replaceOtherComponent(j, source, componentName, options.relative);
        });
    }
}
exports.replaceOther = replaceOther;
