"use strict";
exports.__esModule = true;
exports.replaceTextStyle = void 0;
var jsx_1 = require("../../../utilities/jsx");
var imports_1 = require("../../../utilities/imports");
var constants_1 = require("../../../utilities/constants");
var types_1 = require("../../../utilities/types");
var variationMap = {
    strong: { fontWeight: 'semibold' },
    subdued: { color: 'subdued' },
    positive: { color: 'success' },
    negative: { color: 'critical' },
    warning: { color: 'warning' },
    code: {}
};
/**
 * Replace <TextStyle> with the <Text> component
 */
function replaceTextStyle(j, source, options) {
    var sourcePaths = (0, imports_1.normalizeImportSourcePaths)(j, source, {
        relative: options.relative,
        from: 'TextStyle',
        to: 'Text'
    });
    if (!sourcePaths)
        return;
    if (!(0, imports_1.hasImportSpecifier)(j, source, 'TextStyle', sourcePaths.from) &&
        !(0, imports_1.hasImportSpecifier)(j, source, 'TextStyleProps', sourcePaths.from)) {
        return;
    }
    var localElementName = (0, imports_1.getImportSpecifierName)(j, source, 'TextStyle', sourcePaths.from) ||
        'TextStyle';
    var localElementTypeName = (0, imports_1.getImportSpecifierName)(j, source, 'TextStyleProps', sourcePaths.from);
    var canInsertTextImport = false;
    var canRemoveTextStyleImport = Boolean(!localElementTypeName);
    var canInsertInlineCodeImport = false;
    source.findJSXElements(localElementName).forEach(function (element) {
        var _a;
        var allAttributes = (_a = j(element).find(j.JSXOpeningElement).get().value
            .attributes) !== null && _a !== void 0 ? _a : [];
        if (allAttributes.some(function (attribute) { return attribute.type !== 'JSXAttribute'; })) {
            canRemoveTextStyleImport = false;
            (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
            return;
        }
        var jsxAttributes = allAttributes;
        var variationAttribute = jsxAttributes.find(function (attribute) { return attribute.name.name === 'variation'; });
        var variationValue;
        if (variationAttribute) {
            var variationAttributeValue = variationAttribute.value;
            if ((variationAttributeValue === null || variationAttributeValue === void 0 ? void 0 : variationAttributeValue.type) === 'StringLiteral' &&
                (0, types_1.isKeyOf)(variationMap, variationAttributeValue.value)) {
                variationValue = variationAttributeValue.value;
            }
            else {
                canRemoveTextStyleImport = false;
                (0, jsx_1.insertJSXComment)(j, element, constants_1.POLARIS_MIGRATOR_COMMENT);
                return;
            }
        }
        canInsertTextImport = true;
        (0, jsx_1.replaceJSXElement)(j, element, 'Text');
        (0, jsx_1.insertJSXAttribute)(j, element, 'variant', 'bodyMd');
        (0, jsx_1.insertJSXAttribute)(j, element, 'as', 'span');
        (0, jsx_1.removeJSXAttributes)(j, element, 'variation');
        if (typeof variationValue === 'undefined')
            return;
        if (variationValue === 'code') {
            canInsertInlineCodeImport = true;
            var InlineCode = j.jsxElement(j.jsxOpeningElement(j.jsxIdentifier('InlineCode')), j.jsxClosingElement(j.jsxIdentifier('InlineCode')), element.node.children);
            element.replace(j.jsxElement(element.node.openingElement, element.node.closingElement, [
                InlineCode,
            ]));
            return;
        }
        Object.entries(variationMap[variationValue]).forEach(function (_a) {
            var name = _a[0], value = _a[1];
            (0, jsx_1.insertJSXAttribute)(j, element, name, value);
        });
    });
    source
        .find(j.Identifier)
        .filter(function (path) {
        return path.node.name === localElementName ||
            path.node.name === localElementTypeName;
    })
        .forEach(function (path) {
        if (path.node.type !== 'Identifier')
            return;
        if (path.parent.value.type !== 'ImportSpecifier') {
            canRemoveTextStyleImport = false;
        }
        (0, jsx_1.insertCommentBefore)(j, path, constants_1.POLARIS_MIGRATOR_COMMENT);
        if (path.node.name === localElementName) {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: Text');
        }
        else {
            (0, jsx_1.insertCommentBefore)(j, path, 'Replace with: TextProps');
        }
    });
    if (canInsertTextImport) {
        if (!(0, imports_1.hasImportSpecifier)(j, source, 'Text', sourcePaths.to)) {
            if (options.relative) {
                (0, imports_1.insertImportDeclaration)(j, source, 'Text', sourcePaths.to, sourcePaths.from);
            }
            else {
                (0, imports_1.insertImportSpecifier)(j, source, 'Text', sourcePaths.to);
            }
        }
    }
    var inlineTextSourcePath = options.relative
        ? sourcePaths.from.replace('TextStyle', 'InlineCode')
        : '@shopify/polaris';
    if (canInsertInlineCodeImport) {
        if (!(0, imports_1.hasImportSpecifier)(j, source, 'InlineCode', inlineTextSourcePath)) {
            if (options.relative) {
                (0, imports_1.insertImportDeclaration)(j, source, 'InlineCode', inlineTextSourcePath, sourcePaths.to);
            }
            else {
                (0, imports_1.insertImportSpecifier)(j, source, 'InlineCode', inlineTextSourcePath);
            }
        }
    }
    if (canRemoveTextStyleImport) {
        if ((0, imports_1.hasImportSpecifier)(j, source, 'TextStyle', sourcePaths.from)) {
            (0, imports_1.removeImportSpecifier)(j, source, 'TextStyle', sourcePaths.from);
        }
        if (!(0, imports_1.hasImportSpecifiers)(j, source, sourcePaths.from)) {
            (0, imports_1.removeImportDeclaration)(j, source, sourcePaths.from);
        }
    }
}
exports.replaceTextStyle = replaceTextStyle;
