"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedBorderWidth = (0, sass_1.namespace)('border-width', options);
    return {
        postcssPlugin: 'v9-scss-replace-border-width',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            if (!borderProps.has(decl.prop))
                return;
            /**
             * A collection of transformable values to migrate (e.g. decl lengths, functions, etc.)
             *
             * Note: This is evaluated at the end of each visitor execution to determine whether
             * or not to replace the declaration or insert a comment.
             */
            var targets = [];
            var hasNumericOperator = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            handleBorderProps();
            if (targets.some(function (_a) {
                var replaced = _a.replaced;
                return !replaced || hasNumericOperator;
            })) {
                // Insert comment if the declaration value contains calculations
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            else {
                decl.value = parsedValue.toString();
            }
            //
            // Handlers
            //
            function handleBorderProps() {
                parsedValue.walk(function (node) {
                    var _a, _b, _c;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedBorderWidth, node)) {
                            targets.push({ replaced: false });
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (!(args.length === 0 || args.length === 1))
                                return;
                            // `border-width()` args reference:
                            // https://github.com/shopify/polaris/blob/2b14c0b60097f75d21df7eaa744dfaf84f8f53f7/documentation/guides/legacy-polaris-v8-public-api.scss#L616
                            var value = (_a = args[0]) !== null && _a !== void 0 ? _a : 'base';
                            if (!(0, types_1.isKeyOf)(borderWidthFunctionMap, value))
                                return;
                            node.value = 'var';
                            node.nodes = [
                                {
                                    type: 'word',
                                    value: borderWidthFunctionMap[value],
                                    sourceIndex: (_c = (_b = node.nodes[0]) === null || _b === void 0 ? void 0 : _b.sourceIndex) !== null && _c !== void 0 ? _c : 0,
                                    sourceEndIndex: borderWidthFunctionMap[value].length
                                },
                            ];
                            targets[targets.length - 1].replaced = true;
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
        }
    };
};
var borderProps = new Set([
    'border',
    'border-top',
    'border-right',
    'border-bottom',
    'border-left',
    'border-width',
    'border-top-width',
    'border-right-width',
    'border-bottom-width',
    'border-left-width',
]);
var borderWidthFunctionMap = {
    '': '--p-border-width-1',
    base: '--p-border-width-1',
    "'base'": '--p-border-width-1',
    thick: '--p-border-width-2',
    "'thick'": '--p-border-width-2',
    thicker: '--p-border-width-3',
    "'thicker'": '--p-border-width-3'
};
