"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
exports["default"] = (0, sass_1.createSassMigrator)('v9-replace-sass-transition', function (_, _a, context) {
    var methods = _a.methods, options = _a.options;
    var namespacedDuration = (0, sass_1.namespace)('duration', options);
    return function (root) {
        methods.walkDecls(root, function (decl) {
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            if (!(0, sass_1.hasSassFunction)(namespacedDuration, parsedValue))
                return;
            parsedValue.walk(function (node) {
                var _a, _b;
                if ((0, sass_1.isNumericOperator)(node)) {
                    methods.report({
                        node: decl,
                        severity: 'warning',
                        message: 'Numeric operator detected.'
                    });
                }
                if ((0, sass_1.isSassFunction)(namespacedDuration, node)) {
                    var duration = (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : DEFAULT_DURATION;
                    if (!(0, types_1.isKeyOf)(durationMap, duration)) {
                        methods.report({
                            severity: 'warning',
                            node: decl,
                            message: "Unknown duration key '".concat(duration, "'.")
                        });
                        return sass_1.StopWalkingFunctionNodes;
                    }
                    var durationCustomProperty = durationMap[duration];
                    var targetValue = "var(".concat(durationCustomProperty, ")");
                    if (context.fix) {
                        (0, sass_1.setNodeValue)(node, targetValue);
                    }
                    else {
                        methods.report({
                            severity: 'error',
                            node: decl,
                            message: "Replace duration with token: ".concat(targetValue)
                        });
                    }
                    return sass_1.StopWalkingFunctionNodes;
                }
            });
            if (context.fix) {
                decl.value = parsedValue.toString();
            }
        });
    };
});
var DEFAULT_DURATION = 'base';
var durationMap = {
    none: '--p-duration-0',
    fast: '--p-duration-100',
    base: '--p-duration-200',
    slow: '--p-duration-300',
    slower: '--p-duration-400',
    slowest: '--p-duration-500'
};
