"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
exports["default"] = (0, sass_1.createSassMigrator)('v9-scss-replace-easing', function (_, _a, context) {
    var methods = _a.methods, options = _a.options;
    var namespacedEasing = (0, sass_1.namespace)('easing', options);
    return function (root) {
        methods.walkDecls(root, function (decl) {
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            if (!(0, sass_1.hasSassFunction)(namespacedEasing, parsedValue))
                return;
            parsedValue.walk(function (node) {
                var _a, _b;
                if ((0, sass_1.isNumericOperator)(node)) {
                    methods.report({
                        node: decl,
                        severity: 'warning',
                        message: 'Numeric operator detected.'
                    });
                    return;
                }
                if ((0, sass_1.isSassFunction)(namespacedEasing, node)) {
                    var easing = (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : DEFAULT_EASING;
                    if (!(0, types_1.isKeyOf)(easingMap, easing)) {
                        var comment = deprecatedEasingFuncs.includes(easing)
                            ? "The ".concat(easing, " easing function is no longer available in Polaris. See https://polaris.shopify.com/tokens/motion for possible values.")
                            : "Unexpected easing function '".concat(easing, "'.");
                        methods.report({
                            severity: 'warning',
                            node: decl,
                            message: comment
                        });
                        return sass_1.StopWalkingFunctionNodes;
                    }
                    var easingCustomProperty = easingMap[easing];
                    var targetValue = "var(".concat(easingCustomProperty, ")");
                    if (context.fix) {
                        (0, sass_1.setNodeValue)(node, targetValue);
                    }
                    else {
                        methods.report({
                            severity: 'error',
                            node: decl,
                            message: "Replace easing function with token: ".concat(targetValue)
                        });
                    }
                    return sass_1.StopWalkingFunctionNodes;
                }
            });
            if (context.fix) {
                decl.value = parsedValue.toString();
            }
        });
    };
});
var DEFAULT_EASING = 'base';
var easingMap = {
    base: '--p-ease',
    "in": '--p-ease-in',
    out: '--p-ease-out'
};
var deprecatedEasingFuncs = ['anticipate', 'excite', 'overshoot'];
