"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedFontFamily = (0, sass_1.namespace)('font-family', options);
    return {
        postcssPlugin: 'v9-scss-replace-font-family',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var needsComment = false;
            var needsFix = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            parsedValue.walk(function (node) {
                var _a, _b, _c;
                if ((0, sass_1.isSassFunction)(namespacedFontFamily, node)) {
                    var args = (0, sass_1.getFunctionArgs)(node);
                    // `font-family()` args reference:
                    // https://github.com/shopify/polaris/blob/2b14c0b60097f75d21df7eaa744dfaf84f8f53f7/documentation/guides/legacy-polaris-v8-public-api.scss#L945
                    var family = (_a = args[0]) !== null && _a !== void 0 ? _a : 'base';
                    if (!(0, types_1.isKeyOf)(fontFamilyMap, family)) {
                        needsComment = true;
                        return sass_1.StopWalkingFunctionNodes;
                    }
                    var fontFamilyCustomProperty = fontFamilyMap[family];
                    needsFix = true;
                    node.value = 'var';
                    node.nodes = [
                        {
                            type: 'word',
                            value: fontFamilyCustomProperty,
                            sourceIndex: (_c = (_b = node.nodes[0]) === null || _b === void 0 ? void 0 : _b.sourceIndex) !== null && _c !== void 0 ? _c : 0,
                            sourceEndIndex: fontFamilyCustomProperty.length
                        },
                    ];
                    return sass_1.StopWalkingFunctionNodes;
                }
            });
            if (needsComment) {
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            if (needsFix) {
                decl.value = parsedValue.toString();
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var fontFamilyMap = {
    base: '--p-font-family-sans',
    monospace: '--p-font-family-mono'
};
