"use strict";
/* eslint-disable line-comment-position */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedFontSize = (0, sass_1.namespace)('font-size', options);
    return {
        postcssPlugin: 'v9-scss-replace-font',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var needsFix = false;
            var needsComment = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            parsedValue.walk(function (node) {
                var _a, _b, _c;
                if ((0, sass_1.isSassFunction)(namespacedFontSize, node)) {
                    var args = (0, sass_1.getFunctionArgs)(node);
                    // `font-size()` args reference:
                    // https://github.com/Shopify/polaris/blob/1738f17c739e06dcde4653a9783ca367e38b4e32/documentation/guides/legacy-polaris-v8-public-api.scss#L977
                    var styleArg = args[0];
                    var variantArg = (_a = args[1]) !== null && _a !== void 0 ? _a : 'base';
                    if (!((0, types_1.isKeyOf)(fontSizeMap, styleArg) &&
                        (0, types_1.isKeyOf)(fontSizeMap[styleArg], variantArg))) {
                        needsComment = true;
                        return sass_1.StopWalkingFunctionNodes;
                    }
                    needsFix = true;
                    var fontSizeVariant = fontSizeMap[styleArg][variantArg];
                    if (fontSizeVariant.startsWith('--')) {
                        node.value = 'var';
                        node.nodes = [
                            {
                                type: 'word',
                                value: fontSizeVariant,
                                sourceIndex: (_c = (_b = node.nodes[0]) === null || _b === void 0 ? void 0 : _b.sourceIndex) !== null && _c !== void 0 ? _c : 0,
                                sourceEndIndex: fontSizeVariant.length
                            },
                        ];
                    }
                    else {
                        // @ts-expect-error: We are intentionally changing the node type
                        node.type = 'word';
                        node.value = fontSizeVariant;
                    }
                    return sass_1.StopWalkingFunctionNodes;
                }
            });
            if (needsComment) {
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            if (needsFix) {
                decl.value = parsedValue.toString();
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var fontSizeMap = {
    caption: {
        base: '0.8125rem',
        'large-screen': '--p-font-size-75'
    },
    heading: {
        base: '1.0625rem',
        'large-screen': '--p-font-size-200'
    },
    subheading: {
        base: '0.8125rem',
        'large-screen': '--p-font-size-75'
    },
    input: {
        base: '--p-font-size-200',
        'large-screen': '--p-font-size-100'
    },
    body: {
        base: '0.9375rem',
        'large-screen': '--p-font-size-100'
    },
    button: {
        base: '0.9375rem',
        'large-screen': '--p-font-size-100'
    },
    'button-large': {
        base: '1.0625rem',
        'large-screen': '--p-font-size-200'
    },
    'display-x-large': {
        base: '1.6875rem',
        'large-screen': '2.625rem'
    },
    'display-large': {
        base: '--p-font-size-400',
        'large-screen': '--p-font-size-500'
    },
    'display-medium': {
        base: '1.3125rem',
        'large-screen': '1.625rem'
    },
    'display-small': {
        base: '--p-font-size-200',
        'large-screen': '--p-font-size-300'
    }
};
