"use strict";
/* eslint-disable line-comment-position */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedLineHeight = (0, sass_1.namespace)('line-height', options);
    return {
        postcssPlugin: 'v9-scss-replace-line-height',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var needsFix = false;
            var needsComment = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            parsedValue.walk(function (node) {
                var _a, _b, _c;
                if ((0, sass_1.isSassFunction)(namespacedLineHeight, node)) {
                    var args = (0, sass_1.getFunctionArgs)(node);
                    // `line-height()` args reference:
                    // https://github.com/shopify/polaris/blob/2b14c0b60097f75d21df7eaa744dfaf84f8f53f7/documentation/guides/legacy-polaris-v8-public-api.scss#L961
                    var styleArg = args[0];
                    var variantArg = (_a = args[1]) !== null && _a !== void 0 ? _a : 'base';
                    if (!((0, types_1.isKeyOf)(lineHeightMap, styleArg) &&
                        (0, types_1.isKeyOf)(lineHeightMap[styleArg], variantArg))) {
                        needsComment = true;
                        return sass_1.StopWalkingFunctionNodes;
                    }
                    needsFix = true;
                    var lineHeightVariant = lineHeightMap[styleArg][variantArg];
                    if (lineHeightVariant.startsWith('--')) {
                        node.value = 'var';
                        node.nodes = [
                            {
                                type: 'word',
                                value: lineHeightVariant,
                                sourceIndex: (_c = (_b = node.nodes[0]) === null || _b === void 0 ? void 0 : _b.sourceIndex) !== null && _c !== void 0 ? _c : 0,
                                sourceEndIndex: lineHeightVariant.length
                            },
                        ];
                    }
                    else {
                        // @ts-expect-error: We are intentionally changing the node type
                        node.type = 'word';
                        node.value = lineHeightVariant;
                    }
                    return sass_1.StopWalkingFunctionNodes;
                }
            });
            if (needsComment) {
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            if (needsFix) {
                decl.value = parsedValue.toString();
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var lineHeightMap = {
    caption: {
        base: '--p-font-line-height-2',
        'large-screen': '--p-font-line-height-1'
    },
    heading: {
        base: '--p-font-line-height-3'
    },
    subheading: {
        base: '--p-font-line-height-1'
    },
    input: {
        base: '--p-font-line-height-3'
    },
    body: {
        base: '--p-font-line-height-2'
    },
    button: {
        base: '--p-font-line-height-1'
    },
    'button-large': {
        base: '--p-font-line-height-2'
    },
    'display-x-large': {
        base: '2.25rem',
        'large-screen': '2.75rem'
    },
    'display-large': {
        base: '--p-font-line-height-4',
        'large-screen': '--p-font-line-height-5'
    },
    'display-medium': {
        base: '--p-font-line-height-4',
        'large-screen': '--p-font-line-height-5'
    },
    'display-small': {
        base: '--p-font-line-height-3',
        'large-screen': '--p-font-line-height-4'
    }
};
