"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(file, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(file.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedSpacing = (0, sass_1.namespace)('spacing', options);
    return {
        postcssPlugin: 'v9-replace-sass-spacing',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            // Convert -#{spacing()} to -1 * #{spacing()}
            if ((0, sass_1.hasNegativeSassInterpolation)(decl.value)) {
                (0, sass_1.replaceNegativeSassInterpolation)(parsedValue);
            }
            // Remove #{} from spacing()
            if ((0, sass_1.hasSassInterpolation)(parsedValue.toString())) {
                (0, sass_1.removeSassInterpolation)(namespacedSpacing, parsedValue);
            }
            // Now we can check if the value is a spacing() function
            if (!(0, sass_1.hasSassFunction)(namespacedSpacing, parsedValue))
                return;
            parsedValue.walk(function (node) {
                var _a, _b, _c, _d;
                if (!(0, sass_1.isSassFunction)(namespacedSpacing, node))
                    return;
                var spacing = (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : '';
                if (!(0, types_1.isKeyOf)(spacingMap, spacing))
                    return;
                var spacingCustomProperty = spacingMap[spacing];
                node.value = 'var';
                node.nodes = [
                    {
                        type: 'word',
                        value: spacingCustomProperty,
                        sourceIndex: (_d = (_c = node.nodes[0]) === null || _c === void 0 ? void 0 : _c.sourceIndex) !== null && _d !== void 0 ? _d : 0,
                        sourceEndIndex: spacingCustomProperty.length
                    },
                ];
            });
            if ((0, sass_1.hasNumericOperator)(parsedValue)) {
                // Insert comment if the declaration value contains calculations
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            else {
                decl.value = parsedValue.toString();
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var spacingMap = {
    none: '--p-space-0',
    'extra-tight': '--p-space-1',
    tight: '--p-space-2',
    'base-tight': '--p-space-3',
    '': '--p-space-4',
    base: '--p-space-4',
    loose: '--p-space-5',
    'extra-loose': '--p-space-8'
};
