"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var sass_1 = require("../../utilities/sass");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedZIndex = (0, sass_1.namespace)('z-index', options);
    var namespacedFixedElementStackingOrder = (0, sass_1.namespace)('$fixed-element-stacking-order', options);
    return {
        postcssPlugin: 'v9-scss-replace-z-index',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            if (!(0, sass_1.hasSassFunction)(namespacedZIndex, parsedValue))
                return;
            var containsUnknownSecondArgument = false;
            parsedValue.walk(function (node) {
                var _a, _b, _c, _d, _e, _f;
                if (!(0, sass_1.isSassFunction)(namespacedZIndex, node))
                    return;
                if (hasMoreThanOneArgument(node)) {
                    // If there's more than one argument to the zIndex fn
                    // We assume they're passing in a custom map
                    // In this case its unlikely this will resolve to a polaris token value
                    // transform legacy zIndex usage to map-get and move on.
                    var _g = node.nodes, key = _g[0], _ = _g[1], map = _g[2];
                    if (map.value === namespacedFixedElementStackingOrder &&
                        isValidElement(key.value, fixedElementStackingOrder)) {
                        var fixedElementStackingOrderToken = fixedElementStackingOrder[key.value];
                        node.value = 'var';
                        node.nodes = [
                            {
                                type: 'word',
                                value: fixedElementStackingOrderToken,
                                sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                sourceEndIndex: fixedElementStackingOrderToken.length
                            },
                        ];
                    }
                    else {
                        // map.get arguments are in the reverse order to z-index arguments.
                        // map.get expects the map object first, and the key second.
                        containsUnknownSecondArgument = true;
                        node.value = 'map.get';
                        node.nodes.reverse();
                    }
                }
                else {
                    var element = (_d = (_c = node.nodes[0]) === null || _c === void 0 ? void 0 : _c.value) !== null && _d !== void 0 ? _d : '';
                    if (!isValidElement(element, zIndexMap))
                        return;
                    var zIndexCustomProperty = zIndexMap[element];
                    node.value = 'var';
                    node.nodes = [
                        {
                            type: 'word',
                            value: zIndexCustomProperty,
                            sourceIndex: (_f = (_e = node.nodes[0]) === null || _e === void 0 ? void 0 : _e.sourceIndex) !== null && _f !== void 0 ? _f : 0,
                            sourceEndIndex: zIndexCustomProperty.length
                        },
                    ];
                }
            });
            if ((0, sass_1.hasNumericOperator)(parsedValue) || containsUnknownSecondArgument) {
                // Insert comment if the declaration value contains calculations
                // or if the invocation of zIndex has more than one argument
                decl.before(postcss_1["default"].comment({ text: constants_1.POLARIS_MIGRATOR_COMMENT }));
                decl.before(postcss_1["default"].comment({ text: "".concat(decl.prop, ": ").concat(parsedValue.toString(), ";") }));
            }
            else {
                decl.value = parsedValue.toString();
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var zIndexMap = {
    content: '--p-z-1',
    overlay: '--p-z-2'
};
var fixedElementStackingOrder = {
    'global-ribbon': '--p-z-3',
    'top-bar': '--p-z-4',
    'context-bar': '--p-z-5',
    'small-screen-loading-bar': '--p-z-6',
    'nav-backdrop': '--p-z-7',
    nav: '--p-z-8',
    'skip-to-content': '--p-z-9',
    backdrop: '--p-z-10',
    modal: '--p-z-11',
    toast: '--p-z-12'
};
function isValidElement(element, mapObj) {
    return Object.keys(mapObj).includes(element);
}
var hasMoreThanOneArgument = function (node) { return node.nodes.length > 1; };
