"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var polaris_tokens_1 = require("@shopify/polaris-tokens");
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedRem = (0, sass_1.namespace)('rem', options);
    return {
        postcssPlugin: 'v9-styles-tokenize-font',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            if (decl.parent &&
                decl.parent.type === 'atrule' &&
                decl.parent.name === 'font-face') {
                return;
            }
            var handlers = {
                'font-size': handleFontSize,
                'font-weight': handleFontWeight,
                'line-height': handleFontLineHeight
            };
            if (!(0, types_1.isKeyOf)(handlers, decl.prop))
                return;
            /**
             * A collection of transformable values to migrate (e.g. decl lengths, functions, etc.)
             *
             * Note: This is evaluated at the end of each visitor execution to determine whether
             * or not to replace the declaration or insert a comment.
             */
            var targets = [];
            var hasNumericOperator = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            handlers[decl.prop]();
            if (targets.some(function (_a) {
                var replaced = _a.replaced;
                return !replaced || hasNumericOperator;
            })) {
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            else {
                decl.value = parsedValue.toString();
            }
            //
            // Handlers
            //
            function handleFontSize() {
                parsedValue.walk(function (node) {
                    var _a, _b;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        var dimension = postcss_value_parser_1["default"].unit(node.value);
                        if (!(0, sass_1.isTransformableLength)(dimension))
                            return;
                        targets.push({ replaced: false });
                        var fontSizeInPx = (0, sass_1.isUnitlessZero)(dimension)
                            ? "".concat(dimension.number, "px")
                            : (0, polaris_tokens_1.toPx)("".concat(dimension.number).concat(dimension.unit));
                        if (!(0, types_1.isKeyOf)(fontSizeMap, fontSizeInPx))
                            return;
                        targets[targets.length - 1].replaced = true;
                        node.value = "var(".concat(fontSizeMap[fontSizeInPx], ")");
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedRem, node)) {
                            targets.push({ replaced: false });
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (args.length !== 1)
                                return;
                            var fontSizeInPx = (0, sass_1.toTransformablePx)(args[0]);
                            if (!(0, types_1.isKeyOf)(fontSizeMap, fontSizeInPx))
                                return;
                            targets[targets.length - 1].replaced = true;
                            node.value = 'var';
                            node.nodes = [
                                {
                                    type: 'word',
                                    value: fontSizeMap[fontSizeInPx],
                                    sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                    sourceEndIndex: fontSizeMap[fontSizeInPx].length
                                },
                            ];
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
            function handleFontWeight() {
                parsedValue.walk(function (node) {
                    if (node.type === 'function')
                        return sass_1.StopWalkingFunctionNodes;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        targets.push({ replaced: false });
                        var fontWeight = node.value;
                        if (!(0, types_1.isKeyOf)(fontWeightMap, fontWeight))
                            return;
                        targets[targets.length - 1].replaced = true;
                        node.value = "var(".concat(fontWeightMap[fontWeight], ")");
                    }
                });
            }
            function handleFontLineHeight() {
                parsedValue.walk(function (node) {
                    var _a, _b;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        targets.push({ replaced: false });
                        var lineHeighInPx = (0, sass_1.toTransformablePx)(node.value);
                        if (!(0, types_1.isKeyOf)(fontLineHeightMap, lineHeighInPx))
                            return;
                        targets[targets.length - 1].replaced = true;
                        node.value = "var(".concat(fontLineHeightMap[lineHeighInPx], ")");
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedRem, node)) {
                            targets.push({ replaced: false });
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (args.length !== 1)
                                return;
                            var lineHeightInPx = (0, sass_1.toTransformablePx)(args[0]);
                            if (!(0, types_1.isKeyOf)(fontLineHeightMap, lineHeightInPx))
                                return;
                            targets[targets.length - 1].replaced = true;
                            node.value = 'var';
                            node.nodes = [
                                {
                                    type: 'word',
                                    value: fontLineHeightMap[lineHeightInPx],
                                    sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                    sourceEndIndex: fontLineHeightMap[lineHeightInPx].length
                                },
                            ];
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
var globalValues = new Set(['inherit', 'initial', 'unset']);
var fontSizeMap = {
    '12px': '--p-font-size-75',
    '14px': '--p-font-size-100',
    '16px': '--p-font-size-200',
    '20px': '--p-font-size-300',
    '24px': '--p-font-size-400',
    '28px': '--p-font-size-500',
    '32px': '--p-font-size-600',
    '40px': '--p-font-size-700'
};
var fontLineHeightMap = {
    '16px': '--p-font-line-height-1',
    '20px': '--p-font-line-height-2',
    '24px': '--p-font-line-height-3',
    '28px': '--p-font-line-height-4',
    '32px': '--p-font-line-height-5',
    '40px': '--p-font-line-height-6',
    '48px': '--p-font-line-height-7'
};
var fontWeightMap = {
    400: '--p-font-weight-regular',
    500: '--p-font-weight-medium',
    600: '--p-font-weight-semibold',
    700: '--p-font-weight-bold',
    // https://drafts.csswg.org/css-fonts-3/#propdef-font-weight
    // 100 - Thin
    // 200 - Extra Light (Ultra Light)
    // 300 - Light
    // 400 - Normal
    normal: '--p-font-weight-regular',
    // 500 - Medium
    // 600 - Semi Bold (Demi Bold)
    // 700 - Bold
    bold: '--p-font-weight-bold'
};
