"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var constants_1 = require("../../utilities/constants");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(fileInfo, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(fileInfo.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedRem = (0, sass_1.namespace)('rem', options);
    return {
        postcssPlugin: 'v9-styles-tokenize-shape',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            var handler;
            if (borderProps.has(decl.prop))
                handler = handleBorderProps;
            else if (borderRadiusProps.has(decl.prop))
                handler = handleBorderRadiusProps;
            else
                return;
            /**
             * A collection of transformable values to migrate (e.g. decl lengths, functions, etc.)
             *
             * Note: This is evaluated at the end of each visitor execution to determine whether
             * or not to replace the declaration or insert a comment.
             */
            var targets = [];
            var hasNumericOperator = false;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            handler();
            if (targets.some(function (_a) {
                var replaced = _a.replaced;
                return !replaced || hasNumericOperator;
            })) {
                // Insert comment if the declaration value contains calculations
                decl.before((0, sass_1.createInlineComment)(constants_1.POLARIS_MIGRATOR_COMMENT));
                decl.before((0, sass_1.createInlineComment)("".concat(decl.prop, ": ").concat(parsedValue.toString(), ";")));
            }
            else {
                decl.value = parsedValue.toString();
            }
            //
            // Handlers
            //
            function handleBorderProps() {
                parsedValue.walk(function (node) {
                    var _a, _b;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        var dimension = postcss_value_parser_1["default"].unit(node.value);
                        if (!(0, sass_1.isTransformableLength)(dimension))
                            return;
                        targets.push({ replaced: false });
                        var valueInPx = (0, sass_1.toTransformablePx)(node.value);
                        if (!(0, types_1.isKeyOf)(borderWidthLengthMap, valueInPx))
                            return;
                        node.value = "var(".concat(borderWidthLengthMap[valueInPx], ")");
                        targets[targets.length - 1].replaced = true;
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedRem, node)) {
                            targets.push({ replaced: false });
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (args.length !== 1)
                                return;
                            var valueInPx = (0, sass_1.toTransformablePx)(args[0]);
                            if (!(0, types_1.isKeyOf)(borderWidthLengthMap, valueInPx))
                                return;
                            node.value = 'var';
                            node.nodes = [
                                {
                                    type: 'word',
                                    value: borderWidthLengthMap[valueInPx],
                                    sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                    sourceEndIndex: borderWidthLengthMap[valueInPx].length
                                },
                            ];
                            targets[targets.length - 1].replaced = true;
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
            function handleBorderRadiusProps() {
                parsedValue.walk(function (node) {
                    var _a, _b;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        hasNumericOperator = true;
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        var dimension = postcss_value_parser_1["default"].unit(node.value);
                        if (!(0, sass_1.isTransformableLength)(dimension))
                            return;
                        targets.push({ replaced: false });
                        var valueInPx = (0, sass_1.toTransformablePx)(node.value);
                        if (!(0, types_1.isKeyOf)(borderRadiusLengthMap, valueInPx))
                            return;
                        node.value = "var(".concat(borderRadiusLengthMap[valueInPx], ")");
                        targets[targets.length - 1].replaced = true;
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedRem, node)) {
                            targets.push({ replaced: false });
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (args.length !== 1)
                                return;
                            var valueInPx = (0, sass_1.toTransformablePx)(args[0]);
                            if (!(0, types_1.isKeyOf)(borderRadiusLengthMap, valueInPx))
                                return;
                            node.value = 'var';
                            node.nodes = [
                                {
                                    type: 'word',
                                    value: borderRadiusLengthMap[valueInPx],
                                    sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                    sourceEndIndex: borderRadiusLengthMap[valueInPx].length
                                },
                            ];
                            targets[targets.length - 1].replaced = true;
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
        }
    };
};
var globalValues = new Set(['inherit', 'initial', 'unset']);
var borderProps = new Set([
    'border',
    'border-top',
    'border-right',
    'border-bottom',
    'border-left',
    'border-width',
    'border-top-width',
    'border-right-width',
    'border-bottom-width',
    'border-left-width',
]);
var borderRadiusProps = new Set([
    'border-radius',
    'border-top-left-radius',
    'border-top-right-radius',
    'border-bottom-left-radius',
    'border-bottom-right-radius',
]);
var borderWidthLengthMap = {
    '1px': '--p-border-width-1',
    '2px': '--p-border-width-2',
    '3px': '--p-border-width-3',
    '4px': '--p-border-width-4',
    '5px': '--p-border-width-5'
};
var borderRadiusLengthMap = {
    '2px': '--p-border-radius-05',
    '4px': '--p-border-radius-1',
    '8px': '--p-border-radius-2',
    '12px': '--p-border-radius-3',
    '16px': '--p-border-radius-4',
    '20px': '--p-border-radius-5',
    '30px': '--p-border-radius-6',
    '3px': '--p-border-radius-base',
    '6px': '--p-border-radius-large'
};
