"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
exports["default"] = (0, sass_1.createSassMigrator)('v9-styles-tokenize-space', function (_, _a, context) {
    var methods = _a.methods, options = _a.options;
    var namespacedRem = (0, sass_1.namespace)('rem', options);
    return function (root) {
        methods.walkDecls(root, function (decl) {
            if (!spaceProps.has(decl.prop))
                return;
            var parsedValue = (0, postcss_value_parser_1["default"])(decl.value);
            handleSpaceProps();
            if (context.fix) {
                decl.value = parsedValue.toString();
            }
            function handleSpaceProps() {
                parsedValue.walk(function (node) {
                    var _a, _b;
                    if ((0, sass_1.isNumericOperator)(node)) {
                        methods.report({
                            node: decl,
                            severity: 'warning',
                            message: 'Numeric operator detected.'
                        });
                        return;
                    }
                    if (node.type === 'word') {
                        if (globalValues.has(node.value))
                            return;
                        var dimension = postcss_value_parser_1["default"].unit(node.value);
                        if (!(0, sass_1.isTransformableLength)(dimension))
                            return;
                        var valueInPx = (0, sass_1.toTransformablePx)(node.value);
                        if (!(0, types_1.isKeyOf)(spaceMap, valueInPx)) {
                            methods.report({
                                node: decl,
                                severity: 'error',
                                message: "Non-tokenizable value '".concat(node.value, "'")
                            });
                            return;
                        }
                        if (context.fix) {
                            node.value = "var(".concat(spaceMap[valueInPx], ")");
                            return;
                        }
                        methods.report({
                            node: decl,
                            severity: 'error',
                            message: "Prefer var(".concat(spaceMap[valueInPx], ") Polaris token.")
                        });
                        return;
                    }
                    if (node.type === 'function') {
                        if ((0, sass_1.isSassFunction)(namespacedRem, node)) {
                            var args = (0, sass_1.getFunctionArgs)(node);
                            if (args.length !== 1) {
                                methods.report({
                                    node: decl,
                                    severity: 'error',
                                    message: "Expected 1 argument, got ".concat(args.length)
                                });
                                return;
                            }
                            var valueInPx = (0, sass_1.toTransformablePx)(args[0]);
                            if (!(0, types_1.isKeyOf)(spaceMap, valueInPx)) {
                                methods.report({
                                    node: decl,
                                    severity: 'error',
                                    message: "Non-tokenizable value '".concat(args[0].trim(), "'")
                                });
                                return;
                            }
                            if (context.fix) {
                                node.value = 'var';
                                node.nodes = [
                                    {
                                        type: 'word',
                                        value: spaceMap[valueInPx],
                                        sourceIndex: (_b = (_a = node.nodes[0]) === null || _a === void 0 ? void 0 : _a.sourceIndex) !== null && _b !== void 0 ? _b : 0,
                                        sourceEndIndex: spaceMap[valueInPx].length
                                    },
                                ];
                                return;
                            }
                            methods.report({
                                node: decl,
                                severity: 'error',
                                message: "Prefer var(".concat(spaceMap[valueInPx], ") Polaris token.")
                            });
                        }
                        return sass_1.StopWalkingFunctionNodes;
                    }
                });
            }
        });
    };
});
var globalValues = new Set(['inherit', 'initial', 'unset']);
var spaceProps = new Set([
    'padding',
    'padding-top',
    'padding-right',
    'padding-bottom',
    'padding-left',
    'padding-inline',
    'padding-inline-start',
    'padding-inline-end',
    'padding-block',
    'padding-block-start',
    'padding-block-end',
    'margin',
    'margin-top',
    'margin-right',
    'margin-bottom',
    'margin-left',
    'margin-inline',
    'margin-inline-start',
    'margin-inline-end',
    'margin-block',
    'margin-block-start',
    'margin-block-end',
    'gap',
    'grid-gap',
    'row-gap',
    'grid-row-gap',
    'column-gap',
    'grid-column-gap',
]);
var spaceMap = {
    '1px': '--p-space-025',
    '2px': '--p-space-05',
    '4px': '--p-space-1',
    '8px': '--p-space-2',
    '12px': '--p-space-3',
    '16px': '--p-space-4',
    '20px': '--p-space-5',
    '24px': '--p-space-6',
    '32px': '--p-space-8',
    '40px': '--p-space-10',
    '48px': '--p-space-12',
    '64px': '--p-space-16',
    '80px': '--p-space-20',
    '96px': '--p-space-24',
    '112px': '--p-space-28',
    '128px': '--p-space-32'
};
