"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
exports.__esModule = true;
exports.updateImports = exports.normalizeImportSourcePaths = exports.removeImportSpecifier = exports.renameImportSpecifier = exports.insertImportSpecifier = exports.hasImportSpecifier = exports.hasImportSpecifiers = exports.getImportSpecifierName = exports.getImportSpecifier = exports.getImportAllSpecifiers = exports.hasDefaultImportSpecifier = exports.getDefaultImportSpecifierName = exports.removeDefaultImportSpecifier = exports.getDefaultImportSpecifier = exports.insertImportDeclaration = exports.renameImportDeclaration = exports.removeImportDeclaration = exports.getRelativeImportDeclarationValue = exports.getRelativeImportDeclaration = exports.getImportDeclaration = exports.hasImportDeclaration = void 0;
function hasImportDeclaration(j, source, sourcePath) {
    return Boolean(source
        .find(j.ImportDeclaration)
        .filter(function (path) { return path.node.source.value === sourcePath; }).length);
}
exports.hasImportDeclaration = hasImportDeclaration;
function getImportDeclaration(j, source, sourcePath) {
    return source
        .find(j.ImportDeclaration)
        .filter(function (path) { return path.node.importKind !== 'type'; })
        .filter(function (path) { return path.node.source.value === sourcePath; });
}
exports.getImportDeclaration = getImportDeclaration;
function getRelativeImportDeclaration(j, source, fileName) {
    if (fileName === void 0) { fileName = ''; }
    var relativeRegex = new RegExp(String.raw(templateObject_1 || (templateObject_1 = __makeTemplateObject(["^[./]*", "$"], ["^[\\.\\/]*", "$"])), fileName));
    return source
        .find(j.ImportDeclaration)
        .filter(function (path) {
        return typeof path.node.source.value === 'string' &&
            relativeRegex.test(path.node.source.value);
    });
}
exports.getRelativeImportDeclaration = getRelativeImportDeclaration;
function getRelativeImportDeclarationValue(j, source, fileName) {
    if (fileName === void 0) { fileName = ''; }
    var declarations = getRelativeImportDeclaration(j, source, fileName);
    if (!declarations.length)
        return null;
    return declarations.nodes()[0].source.value || null;
}
exports.getRelativeImportDeclarationValue = getRelativeImportDeclarationValue;
function removeImportDeclaration(j, source, sourcePath) {
    getImportDeclaration(j, source, sourcePath).remove();
}
exports.removeImportDeclaration = removeImportDeclaration;
function renameImportDeclaration(j, source, sourcePath, newSourcePath) {
    getImportDeclaration(j, source, sourcePath).forEach(function (importDeclaration) {
        return (importDeclaration.node.source = j.stringLiteral(newSourcePath));
    });
}
exports.renameImportDeclaration = renameImportDeclaration;
function insertImportDeclaration(j, source, importSpecifier, sourcePath, afterSourcePath) {
    var isSameModuleSpecifier = sourcePath === afterSourcePath;
    if (isSameModuleSpecifier) {
        insertImportSpecifier(j, source, sourcePath, importSpecifier);
    }
    else {
        getImportDeclaration(j, source, afterSourcePath).insertAfter(j.importDeclaration.from({
            source: j.literal(sourcePath),
            specifiers: [j.importSpecifier(j.identifier(importSpecifier))]
        }));
    }
}
exports.insertImportDeclaration = insertImportDeclaration;
function getDefaultImportSpecifier(j, source, sourcePath) {
    return source
        .find(j.ImportDeclaration)
        .filter(function (path) { return path.node.source.value === sourcePath; })
        .find(j.ImportDefaultSpecifier);
}
exports.getDefaultImportSpecifier = getDefaultImportSpecifier;
function removeDefaultImportSpecifier(j, source, sourcePath) {
    return getDefaultImportSpecifier(j, source, sourcePath).remove();
}
exports.removeDefaultImportSpecifier = removeDefaultImportSpecifier;
function getDefaultImportSpecifierName(j, source, sourcePath) {
    var specifiers = getDefaultImportSpecifier(j, source, sourcePath);
    if (!specifiers.length)
        return null;
    return specifiers.nodes()[0].local.name;
}
exports.getDefaultImportSpecifierName = getDefaultImportSpecifierName;
function hasDefaultImportSpecifier(j, source, sourcePath) {
    return Boolean(getDefaultImportSpecifier(j, source, sourcePath).length);
}
exports.hasDefaultImportSpecifier = hasDefaultImportSpecifier;
function getImportAllSpecifiers(j, source, sourcePath) {
    return source
        .find(j.ImportDeclaration)
        .filter(function (path) { return path.node.source.value === sourcePath; })
        .find(j.ImportSpecifier);
}
exports.getImportAllSpecifiers = getImportAllSpecifiers;
function getImportSpecifier(j, source, specifier, sourcePath) {
    return getImportAllSpecifiers(j, source, sourcePath).filter(function (path) { return path.value.imported.name === specifier; });
}
exports.getImportSpecifier = getImportSpecifier;
function getImportSpecifierName(j, source, specifier, sourcePath) {
    var specifiers = getImportSpecifier(j, source, specifier, sourcePath);
    return specifiers.length > 0 ? specifiers.nodes()[0].local.name : null;
}
exports.getImportSpecifierName = getImportSpecifierName;
function hasImportSpecifiers(j, source, sourcePath) {
    var _a;
    return Boolean((_a = getImportAllSpecifiers(j, source, sourcePath)) === null || _a === void 0 ? void 0 : _a.length);
}
exports.hasImportSpecifiers = hasImportSpecifiers;
function hasImportSpecifier(j, source, specifier, sourcePath) {
    var _a;
    return Boolean((_a = getImportSpecifier(j, source, specifier, sourcePath)) === null || _a === void 0 ? void 0 : _a.length);
}
exports.hasImportSpecifier = hasImportSpecifier;
function insertImportSpecifier(j, source, importSpecifier, sourcePath) {
    getImportDeclaration(j, source, sourcePath).replaceWith(function (declaration) {
        return j.importDeclaration.from({
            source: j.literal(sourcePath),
            specifiers: __spreadArray(__spreadArray([], (declaration.value.specifiers || []).filter(function (item) { return item.type === 'ImportSpecifier' && item.imported != null; }), true), [
                j.importSpecifier(j.identifier(importSpecifier)),
            ], false),
            comments: declaration.value.comments || null
        });
    });
}
exports.insertImportSpecifier = insertImportSpecifier;
function renameImportSpecifier(j, source, specifier, newSpecifier, sourcePath) {
    getImportSpecifier(j, source, specifier, sourcePath).replaceWith(j.importSpecifier(j.identifier(newSpecifier)));
}
exports.renameImportSpecifier = renameImportSpecifier;
function removeImportSpecifier(j, source, specifier, sourcePath) {
    getImportSpecifier(j, source, specifier, sourcePath).remove();
}
exports.removeImportSpecifier = removeImportSpecifier;
function normalizeImportSourcePaths(j, source, options) {
    if (options === void 0) { options = {
        relative: false,
        from: '',
        to: ''
    }; }
    var relative = options.relative, from = options.from, to = options.to;
    var sourcePaths = {
        from: '@shopify/polaris',
        to: '@shopify/polaris'
    };
    if (relative) {
        sourcePaths.from = getRelativeImportDeclarationValue(j, source, from) || '';
        if (!sourcePaths.from)
            return null;
        sourcePaths.to =
            getRelativeImportDeclarationValue(j, source, to) ||
                sourcePaths.from.replace(from, to);
    }
    return sourcePaths;
}
exports.normalizeImportSourcePaths = normalizeImportSourcePaths;
function updateImports(j, source, options) {
    var fromSpecifier = options.fromSpecifier, toSpecifier = options.toSpecifier, fromSourcePath = options.fromSourcePath, toSourcePath = options.toSourcePath;
    // Insert new import
    if (!hasImportDeclaration(j, source, toSourcePath)) {
        insertImportDeclaration(j, source, toSpecifier, toSourcePath, fromSourcePath);
    }
    if (!hasImportSpecifier(j, source, toSpecifier, toSourcePath)) {
        insertImportSpecifier(j, source, toSpecifier, toSourcePath);
    }
    // Remove old import
    if (hasImportSpecifier(j, source, fromSpecifier, fromSourcePath)) {
        removeImportSpecifier(j, source, fromSpecifier, fromSourcePath);
    }
    if (!hasImportSpecifiers(j, source, fromSourcePath)) {
        removeImportDeclaration(j, source, fromSourcePath);
    }
}
exports.updateImports = updateImports;
var templateObject_1;
