"use strict";
exports.__esModule = true;
exports.matchesStringOrRegExp = void 0;
/**
 * Compares a string to a second value that, if it fits a certain convention,
 * is converted to a regular expression before the comparison.
 * If it doesn't fit the convention, then two strings are compared.
 *
 * Any strings starting and ending with `/` are interpreted
 * as regular expressions.
 */
function matchesStringOrRegExp(input, comparison) {
    if (!Array.isArray(input)) {
        return testAgainstStringOrRegExpOrArray(input, comparison);
    }
    for (var _i = 0, input_1 = input; _i < input_1.length; _i++) {
        var inputItem = input_1[_i];
        var testResult = testAgainstStringOrRegExpOrArray(inputItem, comparison);
        if (testResult) {
            return testResult;
        }
    }
    return false;
}
exports.matchesStringOrRegExp = matchesStringOrRegExp;
function testAgainstStringOrRegExpOrArray(value, comparison) {
    if (!Array.isArray(comparison)) {
        return testAgainstStringOrRegExp(value, comparison);
    }
    for (var _i = 0, comparison_1 = comparison; _i < comparison_1.length; _i++) {
        var comparisonItem = comparison_1[_i];
        var testResult = testAgainstStringOrRegExp(value, comparisonItem);
        if (testResult) {
            return testResult;
        }
    }
    return false;
}
function testAgainstStringOrRegExp(value, comparison) {
    // If it's a RegExp, test directly
    if (comparison instanceof RegExp) {
        return comparison.test(value) ? { match: value, pattern: comparison } : false;
    }
    // Check if it's RegExp in a string
    var firstComparisonChar = comparison[0];
    var lastComparisonChar = comparison[comparison.length - 1];
    var secondToLastComparisonChar = comparison[comparison.length - 2];
    var comparisonIsRegex = firstComparisonChar === '/' &&
        (lastComparisonChar === '/' ||
            (secondToLastComparisonChar === '/' && lastComparisonChar === 'i'));
    var hasCaseInsensitiveFlag = comparisonIsRegex && lastComparisonChar === 'i';
    // If so, create a new RegExp from it
    if (comparisonIsRegex) {
        var valueMatches = hasCaseInsensitiveFlag
            ? new RegExp(comparison.slice(1, -2), 'i').test(value)
            : new RegExp(comparison.slice(1, -1)).test(value);
        return valueMatches ? { match: value, pattern: comparison } : false;
    }
    // Otherwise, it's a string. Do a strict comparison
    return value === comparison ? { match: value, pattern: comparison } : false;
}
