"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.setNodeValue = exports.createSassMigrator = exports.createInlineComment = exports.isPolarisVar = exports.isTransformableDuration = exports.transformableDurationUnits = exports.StopWalkingFunctionNodes = exports.toTransformablePx = exports.hasTransformableLength = exports.isTransformableLength = exports.isUnitlessZero = exports.transformableLengthUnits = exports.stripQuotes = exports.getFunctionArgs = exports.removeSassInterpolation = exports.replaceNegativeSassInterpolation = exports.hasNegativeSassInterpolation = exports.hasSassInterpolation = exports.hasSassFunction = exports.isSassFunction = exports.hasSassVariable = exports.isSassVariable = exports.hasNumericOperator = exports.isNumericOperator = exports.namespace = exports.getNamespacePattern = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var polaris_tokens_1 = require("@shopify/polaris-tokens");
var constants_1 = require("./constants");
var defaultNamespace = '';
var polarisCustomPropertyRegEx = new RegExp((0, polaris_tokens_1.getCustomPropertyNames)(polaris_tokens_1.tokens).join('|'));
function getNamespace(options) {
    return (options === null || options === void 0 ? void 0 : options.namespace) || defaultNamespace;
}
function getNamespacePattern(options) {
    var namespace = getNamespace(options);
    return namespace ? String.raw(templateObject_1 || (templateObject_1 = __makeTemplateObject(["", "."], ["", "\\."])), namespace) : defaultNamespace;
}
exports.getNamespacePattern = getNamespacePattern;
function namespace(name, options) {
    var namespace = getNamespace(options);
    return namespace ? "".concat(namespace, ".").concat(name) : name;
}
exports.namespace = namespace;
/**
 * Checks if a `valueParser` node is a [Sass numeric operator](https://sass-lang.com/documentation/operators/numeric)
 */
function isNumericOperator(node) {
    return (node.value === '+' ||
        node.value === '-' ||
        node.value === '*' ||
        node.value === '/' ||
        node.value === '%');
}
exports.isNumericOperator = isNumericOperator;
/**
 * Checks if any descendant `valueParser` node is a numeric operator
 */
function hasNumericOperator(parsedValue, deep) {
    if (deep === void 0) { deep = true; }
    var containsNumericOperator = false;
    parsedValue.walk(function (node) {
        if (isNumericOperator(node)) {
            containsNumericOperator = true;
            return exports.StopWalkingFunctionNodes;
        }
        if (!deep) {
            return exports.StopWalkingFunctionNodes;
        }
    });
    return containsNumericOperator;
}
exports.hasNumericOperator = hasNumericOperator;
/**
 * Checks if a `valueParser` node is a [SASS variable](https://sass-lang.com/documentation/variables)
 */
function isSassVariable(node) {
    return node.type === 'word' && node.value.startsWith('$');
}
exports.isSassVariable = isSassVariable;
/**
 * Checks if any descendant `valueParser` node is a SASS variable
 */
function hasSassVariable(parsedValue, deep) {
    if (deep === void 0) { deep = true; }
    var containsSassVariable = false;
    parsedValue.walk(function (node) {
        if (isSassVariable(node)) {
            containsSassVariable = true;
            return exports.StopWalkingFunctionNodes;
        }
        if (!deep) {
            return exports.StopWalkingFunctionNodes;
        }
    });
    return containsSassVariable;
}
exports.hasSassVariable = hasSassVariable;
/**
 * Checks if a `valueParser` node is a given Sass function
 *
 * @example
 * const namespacedRem = namespace('rem', options);
 *
 * if (isSassFunction(namespacedRem, node)) node // FunctionNode
 */
function isSassFunction(name, node) {
    return node.type === 'function' && node.value === name;
}
exports.isSassFunction = isSassFunction;
/**
 * Checks if any descendant `valueParser` node is a given Sass function
 *
 * @example
 * const namespacedRem = namespace('rem', options);
 *
 * if (!hasSassFunction(namespacedRem, parsedValue)) return;
 */
function hasSassFunction(name, parsedValue, deep) {
    if (deep === void 0) { deep = true; }
    var containsSassFunction = false;
    parsedValue.walk(function (node) {
        if (isSassFunction(name, node)) {
            containsSassFunction = true;
            return exports.StopWalkingFunctionNodes;
        }
        if (!deep) {
            return exports.StopWalkingFunctionNodes;
        }
    });
    return containsSassFunction;
}
exports.hasSassFunction = hasSassFunction;
/**
 * Check whether a string has Sass interpolation
 */
function hasSassInterpolation(string) {
    return /#\{.+?\}/.test(string);
}
exports.hasSassInterpolation = hasSassInterpolation;
/**
 * Check whether a string has negative Sass interpolation
 */
function hasNegativeSassInterpolation(string) {
    return /-#\{.+?\}/.test(string);
}
exports.hasNegativeSassInterpolation = hasNegativeSassInterpolation;
/**
 * Replace negative Sass interpolations with a multiplication operator and a negative number
 *
 * @example
 * // Before
 * -#{spacing()};
 *
 * // After
 * -1 * ${spacing()};
 */
function replaceNegativeSassInterpolation(parsedValue) {
    var newNodes = [];
    parsedValue.walk(function (node, index, nodes) {
        var containsInterpolation = /-#\{.+?/.test(node.value);
        if (!containsInterpolation)
            return;
        node.value = node.value.replace('-#{', '#{');
        var left = index > 0 ? nodes.slice(0, index) : [];
        var right = nodes.length - 1 > index ? nodes.slice(index + 1) : [];
        newNodes = __spreadArray(__spreadArray(__spreadArray([], left, true), [
            { type: 'word', value: '-1' },
            { type: 'space', value: ' ' },
            { type: 'word', value: '*' },
            { type: 'space', value: ' ' },
            node
        ], false), right, true);
    });
    parsedValue.nodes = newNodes;
}
exports.replaceNegativeSassInterpolation = replaceNegativeSassInterpolation;
/**
 * Remove the Sass interpolation from parsedValue
 */
function removeSassInterpolation(namespace, parsedValue) {
    parsedValue.walk(function (node, index, nodes) {
        var regexp = new RegExp("#{".concat(namespace));
        var containsInterpolation = regexp.test(node.value);
        if (containsInterpolation) {
            node.value = node.value.replace(/#\{/, '');
            nodes[index + 1].value = nodes[index + 1].value.replace(/}/, '');
        }
    });
    parsedValue.nodes = parsedValue.nodes.filter(function (node) { return !(node.type === 'word' && node.value === ''); });
}
exports.removeSassInterpolation = removeSassInterpolation;
function getFunctionArgs(node) {
    var args = [];
    var arg = '';
    node.nodes.forEach(function (node) {
        if (node.type === 'div' && node.value === ',') {
            args.push(arg);
            arg = '';
            return;
        }
        arg += postcss_value_parser_1["default"].stringify(node);
    });
    if (arg) {
        args.push(arg);
    }
    return args;
}
exports.getFunctionArgs = getFunctionArgs;
/**
 * Removes surrounding quotes from a string
 * @example
 * const string = '"hello"';
 * stripQuotes(string); // hello
 */
function stripQuotes(string) {
    return string.replace(/^['"]|['"]$/g, '');
}
exports.stripQuotes = stripQuotes;
/**
 * All transformable dimension units. These values are used to determine
 * if a decl.value can be converted to pixels and mapped to a Polaris custom property.
 */
exports.transformableLengthUnits = ['px', 'rem'];
function isUnitlessZero(dimension) {
    return dimension && dimension.unit === '' && dimension.number === '0';
}
exports.isUnitlessZero = isUnitlessZero;
function isTransformableLength(dimension) {
    if (!dimension)
        return false;
    return exports.transformableLengthUnits.includes(dimension.unit);
}
exports.isTransformableLength = isTransformableLength;
function hasTransformableLength(parsedValue) {
    var transformableLength = false;
    parsedValue.walk(function (node) {
        if (node.type === 'word' &&
            isTransformableLength(postcss_value_parser_1["default"].unit(node.value))) {
            transformableLength = true;
        }
    });
    return transformableLength;
}
exports.hasTransformableLength = hasTransformableLength;
function toTransformablePx(value) {
    var dimension = postcss_value_parser_1["default"].unit(value);
    if (!isTransformableLength(dimension))
        return;
    return (0, polaris_tokens_1.toPx)("".concat(dimension.number).concat(dimension.unit));
}
exports.toTransformablePx = toTransformablePx;
/**
 * Exit early and stop traversing descendant nodes:
 * https://www.npmjs.com/package/postcss-value-parser:~:text=Returning%20false%20in%20the%20callback%20will%20prevent%20traversal%20of%20descendent%20nodes
 */
exports.StopWalkingFunctionNodes = false;
/**
 * All transformable duration units. These values are used to determine
 * if a decl.value can be mapped to a Polaris custom property.
 *
 * Note: <time> is a dimension with 's' or 'ms' as the unit:
 * https://w3c.github.io/csswg-drafts/css-values-3/#time-value
 */
exports.transformableDurationUnits = ['s', 'ms'];
function isTransformableDuration(dimension) {
    if (!dimension)
        return false;
    // Zero is the only unitless dimension our duration transforms support
    if (isUnitlessZero(dimension))
        return true;
    return exports.transformableDurationUnits.includes(dimension.unit);
}
exports.isTransformableDuration = isTransformableDuration;
function isPolarisVar(node) {
    var _a, _b, _c;
    return (isSassFunction('var', node) &&
        polarisCustomPropertyRegEx.test((_c = (_b = (_a = node.nodes) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) !== null && _c !== void 0 ? _c : ''));
}
exports.isPolarisVar = isPolarisVar;
function createInlineComment(text) {
    var comment = postcss_1["default"].comment({ text: text.replace(/\s+/gm, ' ').trim() });
    comment.raws.left = ' ';
    comment.raws.inline = true;
    return comment;
}
exports.createInlineComment = createInlineComment;
// Expose a stylelint-like API for creating sass migrators so we can easily
// migrate to that tool in the future.
function convertStylelintRuleToPostcssProcessor(ruleFn) {
    return function (fileInfo, _, options) {
        var plugin = {
            postcssPlugin: ruleFn.ruleName,
            // PostCSS will rewalk the AST every time a declaration/rule/etc is
            // mutated by a plugin. This can be useful in some cases, but in ours we
            // only want single-pass behaviour.
            //
            // This can be avoided in 2 ways:
            //
            // 1) Flagging each declaration as we pass it, then skipping it on
            //    subsequent passes.
            // 2) Using postcss's Once() plugin callback.
            //
            // stylelint also uses `Once()`, so we're able to remove this once we've
            // migrated:
            // https://github.com/stylelint/stylelint/blob/cb425cb/lib/postcssPlugin.js#L22
            Once: function (root, _a) {
                var result = _a.result;
                // NOTE: For fullest compatibility with stylelint, we initialise the
                // rule here _inside_ the postcss Once function just like stylelint
                // does. This means multiple passes can be performed without rules
                // accidentally retaining scoped variables, etc.
                ruleFn(
                // Normally, this comes from stylelint config, but for this shim we
                // just hard-code it, and instead rely on the "seconary" options
                // object for passing through the jscodeshift options.
                true, options, 
                // Also normally comes from styelint via the cli `--fix` flag.
                { fix: true })(root, result);
            }
        };
        return (0, postcss_1["default"])(plugin).process(fileInfo.source, {
            syntax: require('postcss-scss')
        }).css;
    };
}
function createSassMigrator(name, ruleFn) {
    var wrappedRule = (function (primary, secondaryOptions, context) {
        var reports = new Map();
        var addDedupedReport = function (newReport) {
            if (!reports.has(newReport.node)) {
                reports.set(newReport.node, []);
            }
            var reportsForNode = reports.get(newReport.node);
            if (reportsForNode.findIndex(function (existingReport) {
                return existingReport.severity === newReport.severity &&
                    existingReport.message === newReport.message;
            }) === -1) {
                reportsForNode.push(newReport);
            }
        };
        var flushReportsAsComments = function () {
            // @ts-expect-error No idea why TS is complaining here
            for (var _i = 0, reports_1 = reports; _i < reports_1.length; _i++) {
                var _a = reports_1[_i], node = _a[0], reportsForNode = _a[1];
                node.before(createInlineComment(constants_1.POLARIS_MIGRATOR_COMMENT));
                for (var _b = 0, reportsForNode_1 = reportsForNode; _b < reportsForNode_1.length; _b++) {
                    var report = reportsForNode_1[_b];
                    node.before(createInlineComment("".concat(report.severity, ": ").concat(report.message)));
                }
            }
            reports.clear();
        };
        function createWalker(args) {
            var walker = args.walker, serialiseSuggestion = args.serialiseSuggestion;
            return function (node) {
                var oldNode;
                if (context.fix) {
                    oldNode = node.clone();
                }
                var result = walker(node);
                var isPartialFix = context.fix &&
                    reports.has(node) &&
                    node.toString() !== oldNode.toString();
                flushReportsAsComments();
                // Our migrations have an opinion on partial fixes (when multiple
                // issues are found in a single node, and some but not all can be
                // fixed): We dump out the partial fix result as a comment
                // immediately above the node.
                if (isPartialFix) {
                    node.before(createInlineComment(serialiseSuggestion(node)));
                    // Undo changes
                    node.replaceWith(oldNode);
                }
                return result;
            };
        }
        return ruleFn(primary, 
        // We're kind of abusing stylelint's types here since the
        // SecondaryOptions param can take an arbitrary object. But we need a
        // way to pass the methods into the rule function somehow, and this way
        // means less Typescript hackery.
        {
            options: secondaryOptions,
            methods: {
                report: addDedupedReport,
                each: function (root, walker) {
                    root.each(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return node.toString(); }
                    }));
                },
                walk: function (root, walker) {
                    root.walk(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return node.toString(); }
                    }));
                },
                walkAtRules: function (root, walker) {
                    root.walkAtRules(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return "@".concat(node.name, " ").concat(node.params); }
                    }));
                },
                walkComments: function (root, walker) {
                    root.walkComments(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return node.text; }
                    }));
                },
                walkDecls: function (root, walker) {
                    root.walkDecls(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return "".concat(node.prop, ": ").concat(node.value, ";"); }
                    }));
                },
                walkRules: function (root, walker) {
                    root.walkRules(createWalker({
                        walker: walker,
                        serialiseSuggestion: function (node) { return node.selector; }
                    }));
                }
            }
        }, context);
    });
    wrappedRule.ruleName = name;
    wrappedRule.meta = {
        // TODO: link directly to the specific rule
        url: 'https://www.npmjs.com/package/@shopify/stylelint-polaris',
        fixable: true
    };
    return convertStylelintRuleToPostcssProcessor(wrappedRule);
}
exports.createSassMigrator = createSassMigrator;
function setNodeValue(node, value) {
    var sourceIndex = node.sourceIndex;
    var parsedValue = (0, postcss_value_parser_1["default"])(value).nodes[0];
    Object.assign(node, parsedValue);
    // The node we're replacing might be mid-way through a higher-level value
    // string. Eg; 'border: 1px solid', the 'solid' node is 5 characters into the
    // higher-level value, so we need to correct the index here.
    node.sourceIndex += sourceIndex;
    node.sourceEndIndex += sourceIndex;
}
exports.setNodeValue = setNodeValue;
var templateObject_1;
